package org.eclipse.jdt.internal.compiler.apt.dispatch;

import javax.lang.model.element.ElementKind;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.ArrayInitializer;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.ast.MemberValuePair;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.impl.ReferenceContext;
import org.eclipse.jdt.internal.compiler.util.Util;
import org.eclipse.jdt.internal.compiler.apt.model.AnnotationMemberValue;
import org.eclipse.jdt.internal.compiler.apt.model.AnnotationMirrorImpl;
import org.eclipse.jdt.internal.compiler.lookup.AptSourceLocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.apt.model.VariableElementImpl;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.apt.model.ExecutableElementImpl;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.apt.model.TypeElementImpl;
import javax.lang.model.element.AnnotationValue;
import javax.lang.model.element.AnnotationMirror;
import javax.lang.model.element.Element;
import javax.tools.Diagnostic;

public class BaseMessagerImpl
{
    static final String[] NO_ARGUMENTS;
    private static /* synthetic */ int[] $SWITCH_TABLE$javax$lang$model$element$ElementKind;
    private static /* synthetic */ int[] $SWITCH_TABLE$javax$tools$Diagnostic$Kind;
    
    static {
        NO_ARGUMENTS = new String[0];
    }
    
    public static AptProblem createProblem(Diagnostic.Kind kind, CharSequence msg, Element e, AnnotationMirror a, AnnotationValue v) {
        ReferenceContext referenceContext = null;
        Annotation[] elementAnnotations = null;
        int startPosition = 0;
        int endPosition = 0;
        if (e != null) {
            switch ($SWITCH_TABLE$javax$lang$model$element$ElementKind()[e.getKind().ordinal()]) {
                case 2:
                case 3:
                case 4:
                case 5: {
                    TypeElementImpl typeElementImpl = (TypeElementImpl)e;
                    Binding typeBinding = typeElementImpl._binding;
                    if ((typeBinding instanceof SourceTypeBinding)) {
                        SourceTypeBinding sourceTypeBinding = (SourceTypeBinding)typeBinding;
                        TypeDeclaration typeDeclaration = referenceContext = (TypeDeclaration)sourceTypeBinding.scope.referenceContext();
                        elementAnnotations = typeDeclaration.annotations;
                        startPosition = typeDeclaration.sourceStart;
                        endPosition = typeDeclaration.sourceEnd;
                    }
                }
                case 1: {
                    break;
                }
                case 11:
                case 12: {
                    ExecutableElementImpl executableElementImpl = (ExecutableElementImpl)e;
                    Binding binding = executableElementImpl._binding;
                    if ((binding instanceof MethodBinding)) {
                        MethodBinding methodBinding = (MethodBinding)binding;
                        AbstractMethodDeclaration sourceMethod = methodBinding.sourceMethod();
                        if (sourceMethod != null) {
                            referenceContext = sourceMethod;
                            elementAnnotations = sourceMethod.annotations;
                            startPosition = sourceMethod.sourceStart;
                            endPosition = sourceMethod.sourceEnd;
                        }
                    }
                }
                case 6: {
                    break;
                }
                case 10: {
                    break;
                }
                case 7:
                case 8: {
                    VariableElementImpl variableElementImpl = (VariableElementImpl)e;
                    Binding binding = variableElementImpl._binding;
                    if ((binding instanceof FieldBinding)) {
                        FieldBinding fieldBinding = (FieldBinding)binding;
                        FieldDeclaration fieldDeclaration = fieldBinding.sourceField();
                        if (fieldDeclaration != null) {
                            ReferenceBinding declaringClass = fieldBinding.declaringClass;
                            if ((declaringClass instanceof SourceTypeBinding)) {
                                SourceTypeBinding sourceTypeBinding2 = (SourceTypeBinding)declaringClass;
                                TypeDeclaration typeDeclaration2 = referenceContext = (TypeDeclaration)sourceTypeBinding2.scope.referenceContext();
                            }
                            elementAnnotations = fieldDeclaration.annotations;
                            startPosition = fieldDeclaration.sourceStart;
                            endPosition = fieldDeclaration.sourceEnd;
                        }
                    }
                    else if ((binding instanceof AptSourceLocalVariableBinding)) {
                        AptSourceLocalVariableBinding parameterBinding = (AptSourceLocalVariableBinding)binding;
                        LocalDeclaration parameterDeclaration = parameterBinding.declaration;
                        if (parameterDeclaration != null) {
                            MethodBinding methodBinding2 = parameterBinding.methodBinding;
                            if (methodBinding2 != null) {
                                referenceContext = methodBinding2.sourceMethod();
                            }
                            elementAnnotations = parameterDeclaration.annotations;
                            startPosition = parameterDeclaration.sourceStart;
                            endPosition = parameterDeclaration.sourceEnd;
                        }
                    }
                }
                case 13:
                case 14: {
                    break;
                }
                case 9: {}
            }
        }
        StringBuilder builder = new StringBuilder();
        if (msg != null) {
            builder.append(msg);
        }
        if (a != null && elementAnnotations != null) {
            AnnotationBinding annotationBinding = ((AnnotationMirrorImpl)a)._binding;
            Annotation annotation = findAnnotation(elementAnnotations, annotationBinding);
            if (annotation != null) {
                startPosition = annotation.sourceStart;
                endPosition = annotation.sourceEnd;
                if (v != null && v instanceof AnnotationMemberValue) {
                    MethodBinding methodBinding3 = ((AnnotationMemberValue)v).getMethodBinding();
                    MemberValuePair[] memberValuePairs = annotation.memberValuePairs();
                    MemberValuePair memberValuePair = null;
                    for (int i = 0; memberValuePair == null && i < memberValuePairs.length; ++i) {
                        if (methodBinding3 == memberValuePairs[i].binding) {
                            memberValuePair = memberValuePairs[i];
                        }
                    }
                    if (memberValuePair != null) {
                        startPosition = memberValuePair.sourceStart;
                        endPosition = memberValuePair.sourceEnd;
                    }
                }
            }
        }
        int lineNumber = 0;
        int columnNumber = 1;
        char[] fileName = null;
        if (referenceContext != null) {
            CompilationResult result = referenceContext.compilationResult();
            fileName = result.fileName;
            int[] lineEnds = null;
            lineNumber = ((startPosition >= 0) ? Util.getLineNumber(startPosition, lineEnds = result.getLineSeparatorPositions(), 0, lineEnds.length - 1) : 0);
            columnNumber = ((startPosition >= 0) ? Util.searchColumnNumber(result.getLineSeparatorPositions(), lineNumber, startPosition) : 0);
        }
        int severity;
        switch ($SWITCH_TABLE$javax$tools$Diagnostic$Kind()[kind.ordinal()]) {
            case 1: {
                severity = 1;
                break;
            }
            default: {
                severity = 0;
                break;
            }
        }
        return new AptProblem(referenceContext, fileName, String.valueOf(builder), 0, BaseMessagerImpl.NO_ARGUMENTS, severity, startPosition, endPosition, lineNumber, columnNumber);
    }
    
    private static Annotation findAnnotation(Annotation[] elementAnnotations, AnnotationBinding annotationBinding) {
        int i = 0;
        while (i < elementAnnotations.length) {
            Annotation annotation = findAnnotation(elementAnnotations[i], annotationBinding);
            if (annotation != null) {
                return annotation;
            }
            ++i;
        }
        return null;
    }
    
    private static Annotation findAnnotation(Annotation elementAnnotation, AnnotationBinding annotationBinding) {
        if (annotationBinding == elementAnnotation.getCompilerAnnotation()) {
            return elementAnnotation;
        }
        for (MemberValuePair mvp : elementAnnotation.memberValuePairs()) {
            Expression v = mvp.value;
            if ((v instanceof Annotation)) {
                Annotation a2 = findAnnotation((Annotation)v, annotationBinding);
                if (a2 != null) {
                    return a2;
                }
            }
            else if ((v instanceof ArrayInitializer)) {
                for (Expression e : ((ArrayInitializer)v).expressions) {
                    if ((e instanceof Annotation)) {
                        Annotation a = findAnnotation((Annotation)e, annotationBinding);
                        if (a != null) {
                            return a;
                        }
                    }
                }
            }
        }
        return null;
    }
    
    public BaseMessagerImpl() {
    }
    
    static /* synthetic */ int[] $SWITCH_TABLE$javax$lang$model$element$ElementKind() {
        int[] $switch_TABLE$javax$lang$model$element$ElementKind = BaseMessagerImpl.$SWITCH_TABLE$javax$lang$model$element$ElementKind;
        if ($switch_TABLE$javax$lang$model$element$ElementKind != null) {
            return $switch_TABLE$javax$lang$model$element$ElementKind;
        }
        int[] $switch_TABLE$javax$lang$model$element$ElementKind2 = new int[ElementKind.values().length];
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.ANNOTATION_TYPE.ordinal()] = 4;
        }
        catch (NoSuchFieldError noSuchFieldError) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.CLASS.ordinal()] = 3;
        }
        catch (NoSuchFieldError noSuchFieldError2) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.CONSTRUCTOR.ordinal()] = 12;
        }
        catch (NoSuchFieldError noSuchFieldError3) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.ENUM.ordinal()] = 2;
        }
        catch (NoSuchFieldError noSuchFieldError4) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.ENUM_CONSTANT.ordinal()] = 6;
        }
        catch (NoSuchFieldError noSuchFieldError5) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.EXCEPTION_PARAMETER.ordinal()] = 10;
        }
        catch (NoSuchFieldError noSuchFieldError6) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.FIELD.ordinal()] = 7;
        }
        catch (NoSuchFieldError noSuchFieldError7) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.INSTANCE_INIT.ordinal()] = 14;
        }
        catch (NoSuchFieldError noSuchFieldError8) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.INTERFACE.ordinal()] = 5;
        }
        catch (NoSuchFieldError noSuchFieldError9) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.LOCAL_VARIABLE.ordinal()] = 9;
        }
        catch (NoSuchFieldError noSuchFieldError10) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.METHOD.ordinal()] = 11;
        }
        catch (NoSuchFieldError noSuchFieldError11) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.OTHER.ordinal()] = 16;
        }
        catch (NoSuchFieldError noSuchFieldError12) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.PACKAGE.ordinal()] = 1;
        }
        catch (NoSuchFieldError noSuchFieldError13) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.PARAMETER.ordinal()] = 8;
        }
        catch (NoSuchFieldError noSuchFieldError14) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.RESOURCE_VARIABLE.ordinal()] = 17;
        }
        catch (NoSuchFieldError noSuchFieldError15) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.STATIC_INIT.ordinal()] = 13;
        }
        catch (NoSuchFieldError noSuchFieldError16) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.TYPE_PARAMETER.ordinal()] = 15;
        }
        catch (NoSuchFieldError noSuchFieldError17) {}
        return BaseMessagerImpl.$SWITCH_TABLE$javax$lang$model$element$ElementKind = $switch_TABLE$javax$lang$model$element$ElementKind2;
    }
    
    static /* synthetic */ int[] $SWITCH_TABLE$javax$tools$Diagnostic$Kind() {
        int[] $switch_TABLE$javax$tools$Diagnostic$Kind = BaseMessagerImpl.$SWITCH_TABLE$javax$tools$Diagnostic$Kind;
        if ($switch_TABLE$javax$tools$Diagnostic$Kind != null) {
            return $switch_TABLE$javax$tools$Diagnostic$Kind;
        }
        int[] $switch_TABLE$javax$tools$Diagnostic$Kind2 = new int[Diagnostic.Kind.values().length];
        try {
            $switch_TABLE$javax$tools$Diagnostic$Kind2[Diagnostic.Kind.ERROR.ordinal()] = 1;
        }
        catch (NoSuchFieldError noSuchFieldError) {}
        try {
            $switch_TABLE$javax$tools$Diagnostic$Kind2[Diagnostic.Kind.MANDATORY_WARNING.ordinal()] = 3;
        }
        catch (NoSuchFieldError noSuchFieldError2) {}
        try {
            $switch_TABLE$javax$tools$Diagnostic$Kind2[Diagnostic.Kind.NOTE.ordinal()] = 4;
        }
        catch (NoSuchFieldError noSuchFieldError3) {}
        try {
            $switch_TABLE$javax$tools$Diagnostic$Kind2[Diagnostic.Kind.OTHER.ordinal()] = 5;
        }
        catch (NoSuchFieldError noSuchFieldError4) {}
        try {
            $switch_TABLE$javax$tools$Diagnostic$Kind2[Diagnostic.Kind.WARNING.ordinal()] = 2;
        }
        catch (NoSuchFieldError noSuchFieldError5) {}
        return BaseMessagerImpl.$SWITCH_TABLE$javax$tools$Diagnostic$Kind = $switch_TABLE$javax$tools$Diagnostic$Kind2;
    }
}
